﻿/*======================================================================================
    Copyright 2021 by TheDummyProgrammer (https://www.thedummyprogrammer.com)

    This file is part of The Dummy Programmer Robot.

    The Dummy Programmer Robot is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    The Dummy Programmer Robot is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with The Dummy Programmer Robot.  If not, see <http://www.gnu.org/licenses/>.
======================================================================================*/

using System;
using TDP.Robot.Core.DynamicData;

namespace TDP.Robot.Core
{
    [Serializable]
    public class ExecutionCondition
    {
        public ExecutionCondition(int objectID, string objectName, string dynamicDataCode, EnumExecutionConditionOperator conditionOperator, 
                                    string minValue, string maxValue)
        {
            ObjectID = objectID;
            ObjectName = objectName;
            DynamicDataCode = dynamicDataCode;
            Operator = conditionOperator;
            MinValue = minValue;
            MaxValue = maxValue;
        }

        public int ObjectID { get; set; }
        public string ObjectName { get; set; }
        public string DynamicDataCode { get; set; }
        public EnumExecutionConditionOperator Operator {get; set; }
        public string MinValue { get; set; }
        public string MaxValue { get; set; }

        public bool EvaluateCondition(ExecResult execResult)
        {
            if (Operator == EnumExecutionConditionOperator.ObjectExecutes && execResult.Result)
                return true;

            if (Operator == EnumExecutionConditionOperator.ObjectDoesNotExecute && !execResult.Result)
                return true;

            if (Operator == EnumExecutionConditionOperator.ValueEqualsTo && execResult.Data[DynamicDataCode].ToString() == MinValue)
                return true;

            if (Operator == EnumExecutionConditionOperator.ValueGreaterThan && (int)execResult.Data[DynamicDataCode] > int.Parse(MinValue))
                return true;

            if (Operator == EnumExecutionConditionOperator.ValueLessThan && (int)execResult.Data[DynamicDataCode] < int.Parse(MinValue))
                return true;

            if (Operator == EnumExecutionConditionOperator.ValueContains && execResult.Data[DynamicDataCode].ToString().Contains(MinValue))
                return true;

            if (Operator == EnumExecutionConditionOperator.ValueStartsWith && execResult.Data[DynamicDataCode].ToString().StartsWith(MinValue))
                return true;

            if (Operator == EnumExecutionConditionOperator.ValueEndsWith && execResult.Data[DynamicDataCode].ToString().EndsWith(MinValue))
                return true;

            if (Operator == EnumExecutionConditionOperator.ValueBetween 
                    && (int)execResult.Data[DynamicDataCode] >= int.Parse(MinValue)
                    && (int)execResult.Data[DynamicDataCode] <= int.Parse(MaxValue)
                )
                return true;

            return false;
        }

        public override string ToString()
        {
            string Result = string.Empty;

            switch (Operator)
            {
                case EnumExecutionConditionOperator.ObjectExecutes:
                    Result = string.Format(Resource.TxtObjectExecutes, ObjectName, ObjectID);
                    break;

                case EnumExecutionConditionOperator.ObjectDoesNotExecute:
                    Result = string.Format(Resource.TxtObjectDoesNotExecute, ObjectName, ObjectID);
                    break;

                case EnumExecutionConditionOperator.ValueEqualsTo:
                    Result = string.Format(Resource.TxtDynamicDataEquals, ObjectName, DynamicDataCode, MinValue);
                    break;

                case EnumExecutionConditionOperator.ValueGreaterThan:
                    Result = string.Format(Resource.TxtDynamicDataGreaterThan, ObjectName, DynamicDataCode, MinValue);
                    break;

                case EnumExecutionConditionOperator.ValueLessThan:
                    Result = string.Format(Resource.TxtDynamicDataLessThan, ObjectName, DynamicDataCode, MinValue);
                    break;

                case EnumExecutionConditionOperator.ValueContains:
                    Result = string.Format(Resource.TxtDynamicDataContains, ObjectName, DynamicDataCode, MinValue);
                    break;

                case EnumExecutionConditionOperator.ValueStartsWith:
                    Result = string.Format(Resource.TxtDynamicDataStartsWith, ObjectName, DynamicDataCode, MinValue);
                    break;

                case EnumExecutionConditionOperator.ValueEndsWith:
                    Result = string.Format(Resource.TxtDynamicDataEndsWith, ObjectName, DynamicDataCode, MinValue);
                    break;

                case EnumExecutionConditionOperator.ValueBetween:
                    Result = string.Format(Resource.TxtDynamicDataBetween, ObjectName, DynamicDataCode, MinValue, MaxValue);
                    break;
            }

            return Result;
        }
    }
}
